using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

using Microsoft.AspNetCore.Builder;
using Microsoft.AspNetCore.Hosting;
using Microsoft.AspNetCore.Mvc;
using Microsoft.Extensions.Configuration;
using Microsoft.Extensions.DependencyInjection;
using Microsoft.Extensions.Hosting;
using Microsoft.Extensions.Logging;
using Microsoft.Extensions.Options;

namespace CorsServer.WebApi31
{
    public class Startup
    {
        public Startup(IConfiguration configuration,IHostEnvironment hostingEnvironment,IWebHostEnvironment webHostEnvironment)
        {
            Configuration = configuration;
        }

        public IConfiguration Configuration { get; }

        public void ConfigureServices(IServiceCollection services)
        {
            #region Config
            services.Configure<CorsOption>(Configuration.GetSection("CORS"));
            #endregion
            #region  CORS
            AddCors_Test(services);
            //AddCors_2(services);
            //AddCors_3(services);
            //AddCors_4(services);
            //AddCors_5(services);
            #endregion
            services.AddControllers();
        }

        public void Configure(IApplicationBuilder app, IWebHostEnvironment env, IOptionsSnapshot<CorsOption> corsOtionsSnapshot)
        {
            if (env.IsDevelopment())
            {
                app.UseDeveloperExceptionPage();
            }

            //·ȫַǰ http://www.custom.com/PathBase/
            //app.UsePathBase("/api/");

            app.UseRouting();

            app.UseCors(CorsPolicyNameConst.DefaultPolicyName);

            app.UseAuthorization();

            app.UseEndpoints(endpoints =>
            {
                endpoints.MapControllers();
            });
        }

        /// <summary>
        /// ȫ˵
        /// </summary>
        private IServiceCollection AddCors_Info(IServiceCollection services)
        {
            services.AddCors(setup =>
            {
                setup.AddPolicy(CorsPolicyNameConst.DefaultPolicyName, build =>
                {
                    build

                    //Դ
                    .AllowAnyOrigin()                               //κԴ
                    //.WithOrigins()                                //ָԴ
                    .SetIsOriginAllowed(_ => true)            //ʹFunc<string bool> ίзȷǷԴ
                    .SetIsOriginAllowedToAllowWildcardSubdomains()  //Դʹͨ(*) 

                    //󷽷(POST GET PUT DELETE OPTIONS)
                    .AllowAnyMethod()                               //з
                    //.WithMethods()                                //ָ

                    //ͷ
                    .AllowAnyHeader()                               //ͷ
                    //.WithHeaders()                                //ָͷ
                    
                    //ƾ
                    .AllowCredentials()                             //ƾݣ֤а(cookies)HTTP֤Э(HTTP authentication schemes)
                    //.DisallowCredentials()                        //ܾƾ

                    //.WithExposedHeaders()                         //ñ¶ԶӦͷĬ£ֻᱩ¶ĬϵӦͷӦãԶӰͷᱩ¶Ӧó
                   ;

                    /*ر˵
                      ڰȫǣ.net core 2.1ʼ, AllowAnyOrigin()  AllowCredentials() ͬʱʹ
                      
                                1ʹAllowCredentials()ʱ.SetIsOriginAllowed(_ => true)  AllowAnyOrigin()
                                2ʹAllowCredentials()ʱ WithOrigins()ָԴ(ʹSetIsOriginAllowedToAllowWildcardSubdomains()ͨ)  AllowAnyOrigin()
                                3Զм
                     */
                });
            });

            return services;
        }

        /// <summary>
        /// CORS ģ
        /// </summary>
        private IServiceCollection AddCors_Template(IServiceCollection services)
        {
            services.AddCors(setup =>
            {
                var corsOption = services.BuildServiceProvider().GetRequiredService<IOptionsSnapshot<CorsOption>>().Value;
                setup.AddPolicy(CorsPolicyNameConst.DefaultPolicyName, build =>
                {
                    build

                    //Դ
                    .AllowAnyOrigin()
                    //.WithOrigins(corsOption.Origins.ToArray())
                    //.SetIsOriginAllowed(_ => true)
                    //.SetIsOriginAllowedToAllowWildcardSubdomains()

                    //󷽷(POST GET PUT DELETE OPTIONS)
                    .AllowAnyMethod()
                    //.WithMethods(corsOption.Methods.ToArray())

                    //ͷ
                    .AllowAnyHeader()
                    //.WithHeaders(corsOption.Headers.ToArray())

                    //ƾ
                    //.AllowCredentials()
                    //.DisallowCredentials()

                    //.WithExposedHeaders()
                   ;
                });
            });

            return services;
        }

        /// <summary>
        /// 
        /// </summary>
        private IServiceCollection AddCors_Test(IServiceCollection services)
        {
            services.AddCors(setup =>
            {
                setup.AddPolicy(CorsPolicyNameConst.DefaultPolicyName, build =>
                {
                    build

                    //Դ
                    .AllowAnyOrigin()
                    //.WithOrigins()
                    //.SetIsOriginAllowed(_ => true)
                    //.SetIsOriginAllowedToAllowWildcardSubdomains()

                    //󷽷(POST GET PUT DELETE OPTIONS)
                    .AllowAnyMethod()
                    //.WithMethods()

                    //ͷ
                    .AllowAnyHeader()
                    //.WithHeaders()

                    //ƾ
                    //.AllowCredentials()
                    //.DisallowCredentials()

                    //.WithExposedHeaders()
                   ;
                });
            });

            return services;
        }

        private IServiceCollection AddCors_Single(IServiceCollection services)
        {
            services.AddCors(setup =>
             {
                 setup.AddPolicy(CorsPolicyNameConst.DefaultPolicyName, build =>
                 {
                     build

                     //Դ
                     .AllowAnyOrigin()
                     //.WithOrigins()
                     //.SetIsOriginAllowed(_ => true)
                     //.SetIsOriginAllowedToAllowWildcardSubdomains()

                     //󷽷(POST GET PUT DELETE OPTIONS)
                     //.AllowAnyMethod()
                     //.WithMethods()

                     //ͷ
                     //.AllowAnyHeader()
                    //.WithHeaders()

                    //ƾ
                    //.AllowCredentials()
                    //.DisallowCredentials()

                    //.WithExposedHeaders()
                    ;
                 });
             });

            return services;
        }

        private IServiceCollection AddCors_All(IServiceCollection services)
        {
            services.AddCors(setup =>
           {
               setup.AddPolicy(CorsPolicyNameConst.DefaultPolicyName, build =>
               {
                   build
                   .AllowAnyOrigin()
                   .AllowAnyMethod()
                   .AllowAnyHeader()
                   ;
               });
           });

            return services;
        }

        private IServiceCollection AddCors_3(IServiceCollection services)
        {
            services.AddCors(setup =>
             {
                 var corsOption = services.BuildServiceProvider().GetRequiredService<IOptionsSnapshot<CorsOption>>().Value;
                 setup.AddPolicy(CorsPolicyNameConst.DefaultPolicyName, build =>
                 {
                     build
                     .WithOrigins(corsOption.Origins.ToArray())
                     .WithMethods(corsOption.Methods.ToArray())
                     .WithHeaders(corsOption.Headers.ToArray())
                     .WithExposedHeaders(corsOption.ExposedHeaders.ToArray());
                 });
             });

            return services;
        }

        private IServiceCollection AddCors_4(IServiceCollection services)
        {
            services.AddCors(setup =>
           {
               setup.AddPolicy(CorsPolicyNameConst.DefaultPolicyName, build =>
               {
                   build.AllowAnyOrigin().AllowAnyMethod().AllowAnyHeader().WithExposedHeaders("x-custom-error");
               });
           });
            return services;
        }

        private IServiceCollection AddCors_5(IServiceCollection services)
        {
            services.AddCors(setup =>
              {
                  var corsOption = services.BuildServiceProvider().GetRequiredService<IOptionsSnapshot<CorsOption>>().Value;
                  setup.AddPolicy(CorsPolicyNameConst.DefaultPolicyName, builder =>
                  {
                      builder
                      ////.SetIsOriginAllowedToAllowWildcardSubdomains()
                      .WithOrigins("http://localhost:5002")
                      //.AllowAnyMethod()
                      ////.WithMethods(corsOption.Methods.ToArray())
                      //.AllowAnyHeader()
                      ////.WithHeaders(corsOption.Headers.ToArray())
                      ////.WithExposedHeaders(corsOption.ExposedHeaders.ToArray())
                      ;
                  });
              });

            return services;
        }
    }
}
